@@ -10,7 +10,6 @@ simulationOutput <- simulateResiduals(fittedModel = fittedModel)
 # informative, but slower
 
 plot(simulationOutput, quantreg = FALSE)
-# idential to plotSimulatedResiduals
 
 #############  qq plot  ######################
 
@@ -30,6 +29,10 @@ plotResiduals(pred = testData$group, residuals = simulationOutput$scaledResidual
               quantreg = FALSE, asFactor = TRUE)
 
 # All these options can also be provided to the main plotting function
+plot(simulationOutput, quantreg = FALSE, rank = FALSE)
+
+# If you want to plot summaries per group, use
+simulationOutput = recalculateResiduals(simulationOutput, group = testData$group)
+plot(simulationOutput, asFactor = TRUE) # we see one residual point per RE
 
-plot(simulationOutput, quantreg = FALSE, rank = TRUE)
 
